@@ -2,7 +2,8 @@
 namespace FSpot {
 	public delegate void AreaUpdatedHandler (object sender, Gdk.Rectangle area);
 	public delegate void AreaPreparedHandler (object sender, System.EventArgs args);
-	
+
+
 	public class AsyncPixbufLoader : System.IDisposable {
 		System.IO.Stream stream;
 		Gdk.PixbufLoader loader;		
@@ -46,7 +47,7 @@ namespace FSpot {
 		public bool Loading
 		{
 			get {
-				return done_reading;
+				return !done_reading;
 			}
 		}
 
@@ -56,6 +57,14 @@ namespace FSpot {
 			}
 		}
 		
+		private void FileLoad (ImageFile img)
+		{
+			pixbuf = img.Load ();
+			done_reading = true;
+			if (Done != null)
+				Done (this, System.EventArgs.Empty);
+		}
+
 		public void Load (string filename)
 		{
 			delay.Stop ();
@@ -68,25 +77,25 @@ namespace FSpot {
 			area_prepared = false;
 			damage = Gdk.Rectangle.Zero;
 
-			try {
-				orientation = PixbufUtils.GetOrientation (filename);
-			} catch (System.Exception e) {
-				System.Console.WriteLine (e.ToString ());
-				orientation = PixbufOrientation.TopLeft;
+			ImageFile img = ImageFile.Create (filename);
+			orientation = img.Orientation;
+
+			stream = img.PixbufStream ();
+			if (stream == null) {
+				FileLoad (img);
+				return;
 			}
 
-			stream = new System.IO.FileStream (filename, System.IO.FileMode.Open, System.IO.FileAccess.Read);
-			
 			loader = new Gdk.PixbufLoader ();
 			loader.AreaPrepared += ap;
 			loader.AreaUpdated += au;
 			loader.Closed += ev;
 
 			ThumbnailGenerator.Default.PushBlock ();
+			//AsyncIORead (null);
 			delay.Start ();
 		}			
 
-
 		public Gdk.PixbufLoader Loader {
 			get {
 				return loader;
@@ -107,28 +116,34 @@ namespace FSpot {
 				; //step
 		}
 
-		private void Close () 
-		{
+		private void Close () {
+			ThumbnailGenerator.Default.PopBlock ();
+				
 			try {
 				delay.Stop ();
-				if (loader != null) {
+				if (loader != null) { 
 					loader.AreaPrepared -= ap;
 					loader.AreaUpdated -= au;
-					loader.Closed -= ev;
+					// this can throw exceptions
 					loader.Close ();
-					loader.Dispose ();
 				}
-
-				loader = null;
 			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
 				if (pixbuf != null)
 					pixbuf.Dispose ();
 
 				pixbuf = null;
 			} finally {
+				if (loader != null) {
+					loader.Closed -= ev;
+					loader.Dispose ();
+				}
+
+				loader = null;
+
 				if (stream != null) 
 					stream.Close ();
-				
+
 				stream = null;
 			}
 		}
@@ -143,8 +158,8 @@ namespace FSpot {
 			if (area.Width != 0 && area.Height != 0 && AreaUpdated != null)
 				AreaUpdated (this, area);
 
+			//System.Console.WriteLine ("orig {0} tform {1}", damage.ToString (), area.ToString ());
 			damage = Gdk.Rectangle.Zero;
-			//System.Console.WriteLine (area.ToString ());
 		}
 
 		private bool AsyncRead () 
@@ -155,15 +170,24 @@ namespace FSpot {
 			do {
 				span = System.DateTime.Now - start_time;
 
-				int len = stream.Read (buffer, 0, buffer.Length);
+				int len;
 				try {
-					loader.Write (buffer, (uint)len);
+					len = stream.Read (buffer, 0, buffer.Length);
+					loader.Write (buffer, (ulong)len);
 				} catch (GLib.GException e) {
+					System.Console.WriteLine (e.ToString ());
 					pixbuf = null;
 					len = -1;
 				}
 
 				if (len <= 0) {
+					if (loader.Pixbuf == null) {
+						if (pixbuf != null)
+							pixbuf.Dispose ();
+
+						pixbuf = null;
+					}
+
 					UpdateListeners ();
 					done_reading = true;
 					Close ();
@@ -191,9 +215,6 @@ namespace FSpot {
 
 		public Gdk.Pixbuf Pixbuf {
 			get {
-				if (!area_prepared)
-					throw new System.Exception ("Load in progress but Pixbuf not ready");
-
 				return pixbuf;
 			}
 		}
