@@ -103,6 +103,7 @@ public class Db : IDisposable {
 	TagStore tag_store;
 	PhotoStore photo_store;
  	ImportStore import_store;
+	bool empty;
 
 	public TagStore Tags {
 		get {
@@ -122,6 +123,16 @@ public class Db : IDisposable {
 		}
 	}
 
+	public bool Sync {
+		set {
+			SqliteCommand command = new SqliteCommand ();
+			command.Connection = sqlite_connection;
+			command.CommandText = "PRAGMA synchronous = " + (value ? "ON" : "OFF");
+			command.ExecuteScalar ();
+			command.Dispose ();
+		}
+	}
+
 	SqliteConnection sqlite_connection;
 
 
@@ -142,6 +153,14 @@ public class Db : IDisposable {
 		tag_store = new TagStore (sqlite_connection, new_db);
 		import_store = new ImportStore (sqlite_connection, new_db);
  		photo_store = new PhotoStore (sqlite_connection, new_db, tag_store);
+
+		empty = new_db;
+	}
+
+	public bool Empty {
+		get {
+			return empty;
+		}
 	}
 
 	public void Dispose () {}
@@ -151,12 +170,12 @@ public class Db : IDisposable {
 public class DbUtils {
 	public static DateTime DateTimeFromUnixTime (uint unix_time)
 	{
-		DateTime date_time = new DateTime (1970, 1, 1);
+		DateTime date_time = new DateTime (1970, 1, 1).ToLocalTime ();
 		return date_time.AddSeconds (unix_time);
 	}
 
 	public static uint UnixTimeFromDateTime (DateTime date_time)
 	{
-		return (uint) (date_time - new DateTime (1970, 1, 1)).TotalSeconds;
+		return (uint) (date_time - new DateTime (1970, 1, 1).ToLocalTime ()).TotalSeconds;
 	}
 }
