@@ -13,7 +13,7 @@ using System.Text.RegularExpressions;
 using LibGPhoto2;
 
 public class MainWindow {
-        public static MainWindow Toplevel = null;
+        public static MainWindow Toplevel;
 
 	Db db;
 
@@ -51,6 +51,15 @@ public class MainWindow {
 	[Glade.Widget] MenuItem update_thumbnail;
 	[Glade.Widget] MenuItem delete_from_drive;
 
+	[Glade.Widget] MenuItem send_mail;
+	[Glade.Widget] MenuItem export;
+	[Glade.Widget] MenuItem print;
+	[Glade.Widget] MenuItem select_none;
+	[Glade.Widget] MenuItem adjust_color;
+	[Glade.Widget] MenuItem exif_data;
+	[Glade.Widget] MenuItem sharpen;
+	[Glade.Widget] MenuItem remove_from_catalog;
+
 	[Glade.Widget] MenuItem display_tags_menu_item;
 
 	[Glade.Widget] MenuItem set_as_background;
@@ -58,14 +67,19 @@ public class MainWindow {
 	[Glade.Widget] MenuItem attach_tag;
 	[Glade.Widget] MenuItem remove_tag;
 	[Glade.Widget] MenuItem find_tag;
+	
+	[Glade.Widget] Scale zoom_scale;
 
 	[Glade.Widget] VPaned info_vpaned;
 
+	[Glade.Widget] Gtk.Image near_image;
+	[Glade.Widget] Gtk.Image far_image;
+
 	PhotoVersionMenu versions_submenu;
 
 	Gtk.ToggleButton browse_button;
 	Gtk.ToggleButton view_button;
-	
+
 	InfoBox info_box;
 	FSpot.InfoDisplay info_display;
 	QueryView icon_view;
@@ -116,8 +130,7 @@ public class MainWindow {
 	{
 		this.db = db;
 
-		Mono.Posix.Catalog.Init ("f-spot", FSpot.Defines.LOCALE_DIR);
-		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", "");
+		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", "f-spot");
 		gui.Autoconnect (this);
 
 		slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
@@ -142,6 +155,7 @@ public class MainWindow {
 		tag_selection_widget.SelectionChanged += OnTagSelectionChanged;
 		tag_selection_widget.DragDataGet += HandleTagSelectionDragDataGet;
 		tag_selection_widget.DragDrop += HandleTagSelectionDragDrop;
+		tag_selection_widget.DragBegin += HandleTagSelectionDragBegin;
 		Gtk.Drag.SourceSet (tag_selection_widget, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
 				    tag_target_table, DragAction.Copy | DragAction.Move);
 
@@ -174,10 +188,15 @@ public class MainWindow {
 		view_vbox.PackStart (group_selector, false, false, 0);
 		view_vbox.ReorderChild (group_selector, 0);
 
+		FSpot.QueryDisplay query_display = new FSpot.QueryDisplay (query, tag_selection_widget);
+		view_vbox.PackStart (query_display, false, false, 0);
+		view_vbox.ReorderChild (query_display, 1);
+
 		icon_view = new QueryView (query);
 		icon_view_scrolled.Add (icon_view);
-		icon_view.SelectionChanged += HandleSelectionChanged;
+		icon_view.Selection.Changed += HandleSelectionChanged;
 		icon_view.DoubleClicked += HandleDoubleClicked;
+		icon_view.Vadjustment.ValueChanged += HandleIconViewScroll;
 		icon_view.GrabFocus ();
 
 		new FSpot.PreviewPopup (icon_view);
@@ -191,6 +210,13 @@ public class MainWindow {
 		TagMenu menu = new TagMenu (attach_tag, db.Tags);
 		menu.TagSelected += HandleAttachTagMenuSelected;
 
+		if (zoom_scale != null) {
+			zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
+		}
+		
+		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
+		far_image.SetFromStock ("f-spot-stock_far", IconSize.SmallToolbar);
+
 		menu = new TagMenu (find_tag, db.Tags);
 		menu.TagSelected += HandleFindTagMenuSelected;
 
@@ -228,13 +254,12 @@ public class MainWindow {
 		main_window.ShowAll ();
 		main_window.Destroyed += HandleCloseCommand;
 
+		query_display.HandleChanged (query);
+
 		if (Toplevel == null)
 			Toplevel = this;
 
 		UpdateToolbar ();
-
-		if (db.Empty)
-			HandleImportCommand (null, null);
 	}
 
 	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
@@ -291,13 +316,15 @@ public class MainWindow {
 			if (view_notebook.CurrentPage != 0)
 				view_notebook.CurrentPage = 0;
 				
-			Present (photo_view.CurrentPhoto);
+			JumpTo (photo_view.Item.Index);
+			zoom_scale.Value = icon_view.ThumbnailWidth / 256.0;
 			break;
 		case ModeType.PhotoView:
 			if (view_notebook.CurrentPage != 1)
 				view_notebook.CurrentPage = 1;
 			
-			Present (icon_view.FocusCell);
+			JumpTo (icon_view.FocusCell);
+			zoom_scale.Value = photo_view.Zoom;
 			break;
 		}
 		UpdateToolbar ();
@@ -319,7 +346,6 @@ public class MainWindow {
 				view_button.Active = state;
 		}
 	}
-		
 
 	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
 	{
@@ -333,21 +359,22 @@ public class MainWindow {
 		}
 	}
 
+	private int lastTopLeftCell = -1;
 
 	public int [] SelectedIds () {
 		int [] ids = new int [0];
 
 		if (fsview != null)
-			ids = new int [] { fsview.View.CurrentPhoto };
+			ids = new int [] { fsview.View.Item.Index };
 		else {
 			switch (view_mode) {
 			case ModeType.IconView:
-				ids = icon_view.SelectedIdxs;
+				ids = icon_view.Selection.Ids;
 				break;
 			default:
 			case ModeType.PhotoView:
-				if (photo_view.View.CurrentPhotoValid ())
-					ids = new int [] { photo_view.CurrentPhoto };
+				if (photo_view.Item.IsValid)
+					ids = new int [] { photo_view.Item.Index };
 				break;
 			}
 		}
@@ -450,6 +477,37 @@ public class MainWindow {
 		}
 	}
 
+	void HandleTagSelectionDragBegin (object sender, DragBeginArgs args)
+	{
+		Tag [] tags = tag_selection_widget.TagHighlight ();
+		int len = tags.Length;
+		int size = 32;
+		int csize = size/2 + len * size / 2 + 2;
+		
+		Pixbuf container = new Pixbuf (Gdk.Colorspace.Rgb, true, 8, csize, csize);
+		container.Fill (0x00000000);
+		
+		bool use_icon = false;;
+		while (len-- > 0) {
+			Pixbuf thumbnail = tags[len].Icon;
+			
+			if (thumbnail != null) {
+				Pixbuf small = PixbufUtils.ScaleToMaxSize (thumbnail, size, size);				
+				
+				int x = len * (size/2) + (size - small.Width)/2;
+				int y = len * (size/2) + (size - small.Height)/2;
+
+				small.Composite (container, x, y, small.Width, small.Height, x, y, 1.0, 1.0, Gdk.InterpType.Nearest, 0xff);
+				small.Dispose ();
+
+				use_icon = true;
+			}
+		}
+		if (use_icon)
+			Gtk.Drag.SetIconPixbuf (args.Context, container, 0, 0);
+		container.Dispose ();
+	}
+	
 	void HandleTagSelectionDragDataGet (object sender, DragDataGetArgs args)
 	{		
 		UriList list = new UriList (SelectedPhotos ());
@@ -538,7 +596,7 @@ public class MainWindow {
 	{
 		switch (view_mode) {
 		case ModeType.PhotoView:
-			photo_view.CurrentPhoto = index;
+			photo_view.Item.Index = index;
 			break;
 		case ModeType.IconView:
 			icon_view.ScrollTo (index);
@@ -552,6 +610,37 @@ public class MainWindow {
 		JumpTo (index);
 	}
 
+	/*
+	 * Keep the glass temporal slider in sync with the user's scrolling in the icon_view
+	 */
+	private void UpdateGlass ()
+	{
+		int cell_num = icon_view.TopLeftVisibleCell();
+		
+		if (cell_num == -1 || cell_num == lastTopLeftCell)
+			return;
+
+		lastTopLeftCell = cell_num;
+		FSpot.IBrowsableItem photo = icon_view.Collection.Items [cell_num];
+#if false
+		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
+		group_selector.Adaptor.SetGlass (group_selector.Adaptor.IndexFromPhoto (photo));
+		group_selector.Adaptor.GlassSet = HandleAdaptorGlassSet;
+#else
+		/* 
+		 * FIXME this is a lame hack to get around a delagate chain.  This should 
+		 * actually operate directly on the adaptor not on the selector but I don't have 
+		 * time to fix it right now.
+		 */
+		group_selector.SetPosition (group_selector.Adaptor.IndexFromPhoto (photo));
+#endif
+	}
+	
+	void HandleIconViewScroll (object sender, EventArgs args)
+	{
+		UpdateGlass ();
+	}
+
 	//
 	// IconView Drag Handlers
 	//
@@ -642,6 +731,14 @@ public class MainWindow {
 		}
 	}
 
+	public void ImportFile (string path)
+	{
+		ImportCommand command = new ImportCommand (main_window);
+		if (command.ImportFromFile (db.Photos, path) > 0) {
+			UpdateQuery ();
+		}
+	}
+
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
@@ -657,7 +754,7 @@ public class MainWindow {
 
 			//Console.WriteLine ("Drop cell = {0} ({1},{2})", item, args.X, args.Y);
 			if (item >= 0) {
-				if (icon_view.CellIsSelected (item))
+				if (icon_view.Selection.Contains (item))
 					AttachTags (tag_selection_widget.TagHighlight (), SelectedIds());
 				else 
 					AttachTags (tag_selection_widget.TagHighlight (), new int [] {item});
@@ -683,7 +780,7 @@ public class MainWindow {
 	// IconView event handlers
 	// 
 
-	void HandleSelectionChanged (IconView view)
+	void HandleSelectionChanged (FSpot.IBrowsableCollection collection)
 	{
 		info_box.Photo = CurrentPhoto;
 		if (info_display != null)
@@ -812,6 +909,11 @@ public class MainWindow {
 	// Main menu commands
 	//
 
+	void HandleOpenCommand (object sender, EventArgs e)
+	{
+		new FSpot.SingleView ();
+	}
+
 	void HandleImportCommand (object sender, EventArgs e)
 	{
 		db.Sync = false;
@@ -824,12 +926,17 @@ public class MainWindow {
 	}
 
 	void HandleImportFromCameraCommand (object sender, EventArgs e)
+	{
+		ImportCamera (null);
+	}
+
+	public void ImportCamera (string camera_device)
 	{
 		GPhotoCamera cam = new GPhotoCamera();
 
 		try {
 			int num_cameras = cam.DetectCameras();
-			int selected_cam;
+			int selected_cam = 0;
 
 			if (num_cameras < 1) {
 				HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
@@ -844,8 +951,21 @@ public class MainWindow {
 			} else if (num_cameras == 1) {
 				selected_cam = 0;
 			} else {
-				FSpot.CameraSelectionDialog camselect = new FSpot.CameraSelectionDialog (cam.CameraList);
-				selected_cam = camselect.Run ();
+				bool found = false;
+				if (camera_device != null) {
+					string port = camera_device.Remove (0, "gphoto2:".Length);
+					for (int i = 0; i < num_cameras; i++)
+						if (cam.CameraList.GetValue (i) == port) {
+							selected_cam = i;
+							found = true;
+							break;
+						}
+				}
+				
+				if (!found) {
+					FSpot.CameraSelectionDialog camselect = new FSpot.CameraSelectionDialog (cam.CameraList);
+					selected_cam = camselect.Run ();
+				}
 			}
 
 			if (selected_cam >= 0) {
@@ -912,11 +1032,6 @@ public class MainWindow {
 		new FSpot.GalleryExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
-	void HandleExportToVfs (object sender, EventArgs args)
-	{
-		new FSpot.VfsExport (new FSpot.PhotoArray (SelectedPhotos ()));
-	}
-
 	void HandleExportToOriginal (object sender, EventArgs args)
 	{
 		new FSpot.FolderExport (new FSpot.PhotoArray (SelectedPhotos ()));
@@ -936,6 +1051,16 @@ public class MainWindow {
 		win.ShowAll ();
 	}
 
+	void HandleViewSelection (object sender, EventArgs args)
+	{
+		Gtk.Window win = new Gtk.Window ("This is a window");
+		Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
+	
+		win.Add (scroll);
+		scroll.Add (new TrayView (icon_view.Selection));
+		win.ShowAll ();
+	}
+
 	void HandleExportToFlickr (object sender, EventArgs args)
 	{
 		new FSpot.FlickrExport (new FSpot.PhotoArray (SelectedPhotos ()));
@@ -959,9 +1084,7 @@ public class MainWindow {
 			url.Append ("&attach=" + p.DefaultVersionPath);
 		}
 
-		Console.WriteLine (url.ToString ());
-
-		Gnome.Url.Show (url.ToString ());
+		GnomeUtil.UrlShow (main_window, url.ToString ());
 	}
 
 	void HandleAbout (object sender, EventArgs args)
@@ -969,6 +1092,7 @@ public class MainWindow {
 		string [] authors = new string [] {
 			"Ettore Perazzoli",
 			"Lawrence Ewing",
+			"Laurence Hygate",
 			"Nat Friedman",
 			"Miguel de Icaza",
 			"Vladimir Vukicevic",
@@ -984,19 +1108,23 @@ public class MainWindow {
 			"Patanjali Somayaji",
 			"Matt Jones",
 			"Martin Willemoes Hansen",
-			"Laurence Hygate"
-
+			"Joerg Buesse",
+			"Jakub Steiner",
+			"Xavier Bouchoux"
 		};
 
                 // Translators should localize the following string
                 // * which will give them credit in the About box.
                 // * E.g. "Martin Willemoes Hansen"
                 string translators = Mono.Posix.Catalog.GetString ("translator-credits");
+                if(System.String.Compare(translators,"translator-credits") == 0) {
+                    translators = null;
+                }
 
-                new About (FSpot.Defines.PACKAGE, 
+                new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
 			   "Copyright 2003-2005 Novell Inc.",
-                           null, authors, null, translators, null).Show();
+                           null, authors, new string [0], translators, null).Show();
 	}
 
 	void HandleArrangeByTime (object sender, EventArgs args)
@@ -1135,8 +1263,8 @@ public class MainWindow {
 		table.Attach (new Gtk.Label (Mono.Posix.Catalog.GetString ("Radius:")), 0, 1, 1, 2);
 		table.Attach (new Gtk.Label (Mono.Posix.Catalog.GetString ("Threshold:")), 0, 1, 2, 3);
 
-		Gtk.SpinButton amount_spin = new Gtk.SpinButton (0.0, 100.0, .01);
-		Gtk.SpinButton radius_spin = new Gtk.SpinButton (0.0, 50.0, .01);
+		Gtk.SpinButton amount_spin = new Gtk.SpinButton (0.5, 100.0, .01);
+		Gtk.SpinButton radius_spin = new Gtk.SpinButton (5.0, 50.0, .01);
 		Gtk.SpinButton threshold_spin = new Gtk.SpinButton (0.0, 50.0, .01);
 
 		table.Attach (amount_spin, 1, 2, 0, 1);
@@ -1159,10 +1287,12 @@ public class MainWindow {
 			
 				Photo photo = query.Photos [id];
 				Exif.ExifData exif_data = new Exif.ExifData (photo.DefaultVersionPath);
-
+				
+				bool created_version = false;
 				uint version = photo.DefaultVersionId;
 				if (version == Photo.OriginalVersionId) {
 					version = photo.CreateDefaultModifiedVersion (photo.DefaultVersionId, false);
+					created_version = true;
 				}
 				
 				try {
@@ -1172,9 +1302,20 @@ public class MainWindow {
 					FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
 					photo.DefaultVersionId = version;
 					query.Commit (id);
-				} catch (GLib.GException ex) {
-					// FIXME error dialog.
-					Console.WriteLine ("error {0}", ex);
+				} catch (System.Exception e) {
+					string msg = Mono.Posix.Catalog.GetString ("Error saving sharpened photo");
+					string desc = String.Format (Mono.Posix.Catalog.GetString ("Received exception \"{0}\". Unable to save image {1}"),
+								     e.Message, photo.Name);
+					
+					HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+										    Gtk.MessageType.Error, ButtonsType.Ok, 
+										    msg,
+										    desc);
+					md.Run ();
+					md.Destroy ();
+					
+					if (created_version)
+						photo.DeleteVersion (version);
 				}
 			
 			}
@@ -1224,6 +1365,11 @@ public class MainWindow {
 			info_vpaned.Show ();
 	}
 
+	public void Present ()
+	{
+		main_window.Present ();
+	}
+	
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
 		main_window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
@@ -1232,9 +1378,6 @@ public class MainWindow {
 
 	private bool SlideShow ()
 	{
-		Gtk.Window win = new Gtk.Window ("test");
-		Pixbuf bg = PixbufUtils.LoadFromScreen ();
-
 		int [] ids = SelectedIds ();
 		Photo [] photos = null;
 		if (ids.Length < 2) {
@@ -1250,52 +1393,19 @@ public class MainWindow {
 		} else {
 			photos = SelectedPhotos ();
 		}
-		
+
 		if (photos.Length == 0) {
 			Console.WriteLine ("No photos available -- no slideshow");
 			main_window.GdkWindow.Cursor = null;
 			return false;
 		}
 
-		SlideView slideview = new SlideView (bg, photos);
-		win.ButtonPressEvent += HandleSlideViewButtonPressEvent;
-		win.KeyPressEvent += HandleSlideViewKeyPressEvent;
-		win.AddEvents ((int) (EventMask.ButtonPressMask | EventMask.KeyPressMask));
-		win.Add (slideview);
-		win.Decorated = false;
-		win.Fullscreen();
-		win.Realize ();
-		win.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
-	
-		Gdk.GCValues values = new Gdk.GCValues ();
-		values.SubwindowMode = SubwindowMode.IncludeInferiors;
-		Gdk.GC fillgc = new Gdk.GC (win.GdkWindow, values, Gdk.GCValuesMask.Subwindow);
-
-		slideview.Show ();
-		win.GdkWindow.SetBackPixmap (null, false);
-		win.Show ();
-		main_window.GdkWindow.Cursor = null;	
-		bg.RenderToDrawable (win.GdkWindow, fillgc, 
-				     0, 0, 0, 0, -1, -1, RgbDither.Normal, 0, 0);
-
-		slideview.Play ();
+		FSpot.FullSlide full = new FSpot.FullSlide (main_window, photos);
+		full.Play ();
+		main_window.GdkWindow.Cursor = null;
 		return false;
 	}
-	
-	[GLib.ConnectBefore]
-	private void HandleSlideViewKeyPressEvent (object sender, KeyPressEventArgs args)
-	{
-		Gtk.Window win = sender as Gtk.Window;
-		win.Destroy ();
-		args.RetVal = true;
-	}
 
-	private void HandleSlideViewButtonPressEvent (object sender, ButtonPressEventArgs args)
-	{
-		Gtk.Window win = sender as Gtk.Window;
-		win.Destroy ();
-		args.RetVal = true;
-	}
 
 	void HandleToggleViewBrowse (object sender, EventArgs args)
 	{
@@ -1336,30 +1446,30 @@ public class MainWindow {
 			fsview.Destroyed += HandleFullScreenViewDestroy;
 		}
 		// FIXME this needs to be another mode like PhotoView and IconView mode.
-		fsview.View.CurrentPhoto = active;
+		fsview.View.Item.Index = active;
 
 		fsview.Show ();
 	}
-	
-	void Present (int item)
-	{
-		switch (view_mode) {
-		case ModeType.IconView:
-			icon_view.ScrollTo (item);
-			icon_view.Throb (item);
-			break;
-		case ModeType.PhotoView:
-			photo_view.CurrentPhoto = item;
-			break;
-		}
-	}
 
 	void HandleFullScreenViewDestroy (object sender, EventArgs args)
 	{
-		Present (fsview.View.CurrentPhoto);
+		JumpTo (fsview.View.Item.Index);
 		fsview = null;
 	}
 	
+	void HandleZoomScaleValueChanged (object sender, System.EventArgs args)
+	{
+		switch (view_mode) {
+		case ModeType.PhotoView:
+			if (System.Math.Abs (photo_view.Zoom - zoom_scale.Value) > System.Double.Epsilon) 
+				photo_view.Zoom = System.Math.Max (0.1, zoom_scale.Value);
+			break;
+		case ModeType.IconView:
+			icon_view.ThumbnailWidth = (int)(System.Math.Max (15, zoom_scale.Value * 256));
+			break;
+		}
+	}
+
 	void HandleZoomOut (object sender, EventArgs args)
 	{
 		switch (view_mode) {
@@ -1464,7 +1574,7 @@ public class MainWindow {
 
 	void HandleSelectNoneCommand (object sender, EventArgs args)
 	{
-		icon_view.UnselectAllCells ();
+		icon_view.Selection.Clear ();
 	}
 	
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
@@ -1524,8 +1634,8 @@ public class MainWindow {
 		}
 		
 		String data = paths.ToString ();
-		primary.SetText (data);
-		clipboard.SetText (data);
+		primary.Text = data;
+		clipboard.Text = data;
 	}
 
 	void HandleSetAsBackgroundCommand (object sender, EventArgs args)
@@ -1586,8 +1696,8 @@ public class MainWindow {
 
 	void OnTagSelectionChanged (object obj)
 	{
-		SetViewMode (ModeType.IconView);
 		UpdateQuery ();
+		SetViewMode (ModeType.IconView);
 	}
 	
 	void HandleTagSelectionChanged (object obj, EventArgs args)
@@ -1597,15 +1707,21 @@ public class MainWindow {
 
 	void HandleQueryItemChanged (FSpot.IBrowsableCollection browsable, int item)
 	{
-		if (info_box.Photo == (browsable.Items[item] as Photo))
+		if (info_box.Photo == (browsable [item] as Photo))
 			info_box.Update ();
+
+		UpdateMenus ();
 	}
 	//
 	// Handle Main Menu 
 
 	void UpdateMenus ()
 	{
-		if (CurrentPhoto == null) {
+		bool tag_sensitive = tag_selection_widget.Selection.CountSelectedRows () > 0;
+		bool active_selection = PhotoSelectionActive ();
+		bool single_active = CurrentPhoto != null;
+		
+		if (!single_active) {
 			version_menu_item.Sensitive = false;
 			version_menu_item.Submenu = new Menu ();
 
@@ -1613,7 +1729,6 @@ public class MainWindow {
 			delete_version_menu_item.Sensitive = false;
 			rename_version_menu_item.Sensitive = false;
 
-			set_as_background.Sensitive = true;
 		} else {
 			version_menu_item.Sensitive = true;
 			create_version_menu_item.Sensitive = true;
@@ -1629,18 +1744,27 @@ public class MainWindow {
 			versions_submenu = new PhotoVersionMenu (CurrentPhoto);
 			versions_submenu.VersionIdChanged += new PhotoVersionMenu.VersionIdChangedHandler (HandleVersionIdChanged);
 			version_menu_item.Submenu = versions_submenu;
-
-			set_as_background.Sensitive = true;
 		}
 
-		bool tag_sensitive = tag_selection_widget.Selection.CountSelectedRows () > 0;
-		bool active_selection = PhotoSelectionActive ();
+		set_as_background.Sensitive = single_active;
+		adjust_color.Sensitive = single_active;
+
+		attach_tag.Sensitive = active_selection;
+		remove_tag.Sensitive = active_selection;
 
 		rotate_left.Sensitive = active_selection;
 		rotate_right.Sensitive = active_selection;
 		update_thumbnail.Sensitive = active_selection;
 		delete_from_drive.Sensitive = active_selection;
 		copy.Sensitive = active_selection;
+		
+		send_mail.Sensitive = active_selection;
+		print.Sensitive = active_selection;
+		export.Sensitive = active_selection;
+		select_none.Sensitive = active_selection;
+		exif_data.Sensitive = active_selection;
+		sharpen.Sensitive = active_selection;
+		remove_from_catalog.Sensitive = active_selection;
 
 		delete_selected_tag.Sensitive = tag_sensitive;
 		edit_selected_tag.Sensitive = tag_sensitive;
